# Generated by Django 3.1 on 2020-09-15 14:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Directory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dn', models.CharField(max_length=100)),
                ('dc', models.GenericIPAddressField()),
                ('username', models.CharField(max_length=100)),
                ('password', models.CharField(max_length=100)),
                ('ldaps', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Query',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=50, null=True)),
                ('query', models.CharField(max_length=200)),
                ('base_dn', models.CharField(blank=True, max_length=200, null=True, verbose_name='Base DN')),
                ('type', models.CharField(choices=[('user', 'User'), ('computer', 'Computer'), ('group', 'Group')], max_length=15)),
                ('target', models.CharField(choices=[('ad_import.Workstation', 'Workstation'), ('costs.Server', 'Server'), ('costs.User', 'User'), ('ad_import.Group', 'Group')], max_length=200)),
                ('directory', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='ad_import.directory')),
            ],
            options={
                'verbose_name': 'LDAP query',
                'verbose_name_plural': 'LDAP queries',
            },
        ),
    ]



from django.core.management.base import BaseCommand

from ad_import.load_data import LoadServers


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('directory', nargs='+', type=str)

    def handle(self, *args, **options):
        directory = options['directory'][0]
        load = LoadServers()
        load.connect(directory)
        for query in load.queries:
            load.load(query)



# Generated by Django 3.1 on 2020-09-17 20:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ad_import', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='directory',
            options={'verbose_name_plural': 'directories'},
        ),
        migrations.AddField(
            model_name='directory',
            name='name',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='query',
            name='directory',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='queries', to='ad_import.directory'),
        ),
        migrations.AlterField(
            model_name='query',
            name='target',
            field=models.CharField(choices=[('ad_import.Workstation', 'Workstation'), ('costs.Server', 'Server'), ('ad_import.User', 'User'), ('ad_import.Group', 'Group')], max_length=200),
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('accountExpires', models.DateTimeField(null=True, verbose_name='konto utløper')),
                ('cn', models.CharField(blank=True, max_length=255, null=True, verbose_name='fullt navn')),
                ('company', models.CharField(blank=True, max_length=255, null=True, verbose_name='firma')),
                ('department', models.CharField(blank=True, max_length=255, null=True, verbose_name='avdeling')),
                ('departmentNumber', models.CharField(blank=True, max_length=255, null=True, verbose_name='avdelingsnummer')),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='beskrivelse')),
                ('displayName', models.CharField(blank=True, max_length=255, null=True, verbose_name='visningsnavn')),
                ('distinguishedName', models.CharField(blank=True, max_length=300, null=True, verbose_name='DN')),
                ('employeeID', models.CharField(blank=True, max_length=255, null=True)),
                ('employeeNumber', models.CharField(blank=True, max_length=255, null=True)),
                ('givenName', models.CharField(blank=True, max_length=255, null=True, verbose_name='fornavn')),
                ('homeDirectory', models.CharField(blank=True, max_length=255, null=True)),
                ('homeDrive', models.CharField(blank=True, max_length=255, null=True)),
                ('ipPhone', models.CharField(blank=True, max_length=255, null=True, verbose_name='IP-telefon')),
                ('lastLogon', models.DateTimeField(blank=True, null=True, verbose_name='siste pålogging')),
                ('logonCount', models.IntegerField(null=True, verbose_name='Antall pålogginger')),
                ('mail', models.EmailField(max_length=254, null=True, verbose_name='Epostadresse')),
                ('middleName', models.CharField(blank=True, max_length=255, null=True, verbose_name='mellomnavn')),
                ('mobile', models.CharField(blank=True, max_length=255, null=True, verbose_name='mobil')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Navn')),
                ('objectGUID', models.CharField(blank=True, max_length=255, null=True)),
                ('objectSid', models.CharField(blank=True, max_length=255, null=True)),
                ('physicalDeliveryOfficeName', models.CharField(blank=True, max_length=255, null=True, verbose_name='Lokasjon')),
                ('pwdLastSet', models.DateTimeField(null=True, verbose_name='passord sist endret')),
                ('sAMAccountName', models.CharField(blank=True, max_length=255, null=True, verbose_name='brukernavn')),
                ('scriptPath', models.CharField(blank=True, max_length=255, null=True)),
                ('sn', models.CharField(blank=True, max_length=255, null=True, verbose_name='etternavn')),
                ('telephoneNumber', models.CharField(blank=True, max_length=255, null=True)),
                ('title', models.CharField(blank=True, max_length=255, null=True)),
                ('userAccountControl', models.IntegerField(null=True)),
                ('userPrincipalName', models.CharField(blank=True, max_length=255, null=True)),
                ('whenChanged', models.DateTimeField(null=True, verbose_name='sist endret')),
                ('whenCreated', models.DateTimeField(null=True, verbose_name='opprettet')),
                ('last_update', models.DateTimeField(auto_now=True, verbose_name='sist oppdatert')),
                ('directory', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='ad_import.directory')),
                ('manager', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='ad_import.user', verbose_name='Leder')),
            ],
        ),
    ]



from django.db import models

from . import Directory


class User(models.Model):
    accountExpires = models.DateTimeField('konto utløper', null=True)
    cn = models.CharField('fullt navn', max_length=255, blank=True, null=True)
    company = models.CharField('firma', max_length=255, blank=True, null=True)
    department = models.CharField('avdeling', max_length=255, blank=True, null=True)
    departmentNumber = models.CharField('avdelingsnummer', max_length=255, blank=True, null=True)
    description = models.CharField('beskrivelse', max_length=255, blank=True, null=True)
    displayName = models.CharField('visningsnavn', max_length=255, blank=True, null=True)
    distinguishedName = models.CharField('DN', max_length=300, blank=True, null=True)
    employeeID = models.CharField(max_length=255, blank=True, null=True)
    employeeNumber = models.CharField(max_length=255, blank=True, null=True)
    givenName = models.CharField('fornavn', max_length=255, blank=True, null=True)
    homeDirectory = models.CharField(max_length=255, blank=True, null=True)
    homeDrive = models.CharField(max_length=255, blank=True, null=True)
    ipPhone = models.CharField('IP-telefon', max_length=255, blank=True, null=True)
    lastLogon = models.DateTimeField('siste pålogging', null=True, blank=True)
    logonCount = models.IntegerField('Antall pålogginger', null=True)
    mail = models.EmailField('Epostadresse', null=True)
    manager = models.ForeignKey('self', on_delete=models.CASCADE, verbose_name='Leder', null=True)
    middleName = models.CharField('mellomnavn', max_length=255, blank=True, null=True)
    mobile = models.CharField('mobil', max_length=255, blank=True, null=True)
    name = models.CharField('Navn', max_length=255, blank=True, null=True)
    objectGUID = models.CharField(max_length=255, blank=True, null=True)
    objectSid = models.CharField(max_length=255, blank=True, null=True)
    physicalDeliveryOfficeName = models.CharField('Lokasjon', max_length=255, blank=True, null=True)
    pwdLastSet = models.DateTimeField('passord sist endret', null=True)
    sAMAccountName = models.CharField('brukernavn', max_length=255, blank=True, null=True)
    scriptPath = models.CharField(max_length=255, blank=True, null=True)
    sn = models.CharField('etternavn', max_length=255, blank=True, null=True)
    telephoneNumber = models.CharField(max_length=255, blank=True, null=True)
    title = models.CharField(max_length=255, blank=True, null=True)
    userAccountControl = models.IntegerField(null=True)
    userPrincipalName = models.CharField(max_length=255, blank=True, null=True)
    whenChanged = models.DateTimeField('sist endret', null=True)
    whenCreated = models.DateTimeField('opprettet', null=True)

    directory = models.ForeignKey(Directory, on_delete=models.PROTECT)
    last_update = models.DateTimeField('sist oppdatert', auto_now=True)

    def __str__(self):
        return '%s' % self.displayName

    def disabled(self):
        return self.userAccountControl & 2 == 2


from typing import Type, Dict, List, Union

import ldap
from ldap.controls import SimplePagedResultsControl

from .paginated_lookup import paginated_query
from . import ADObject

response_types = {
    0: 'success',
    1: 'operationsError',
    2: 'protocolError',
    3: 'timeLimitExceeded',
    4: 'sizeLimitExceeded',
    5: 'compareFalse',
    6: 'compareTrue',
    7: 'authMethodNotSupported',
    8: 'strongerAuthRequired',
    10: 'referral',
    11: 'adminLimitExceeded',
    12: 'unavailableCriticalExtension',
    13: 'confidentialityRequired',
    14: 'saslBindInProgress',
    16: 'noSuchAttribute',
    17: 'undefinedAttributeType',
    18: 'inappropriateMatching',
    19: 'constraintViolation',
    20: 'attributeOrValueExists',
    21: 'invalidAttributeSyntax',
    32: 'noSuchObject',
    33: 'aliasProblem',
    34: 'invalidDNSyntax',
    35: 'isLeaf',
    36: 'aliasDereferencingProblem',
    48: 'inappropriateAuthentication',
    49: 'invalidCredentials',
    50: 'insufficientAccessRights',
    51: 'busy',
    52: 'unavailable',
    53: 'unwillingToPerform',
    54: 'loopDetect',
    60: 'sortControlMissing',
    61: 'offsetRangeError',
    64: 'namingViolation',
    65: 'objectClassViolation',
    66: 'notAllowedOnNonLeaf',
    67: 'notAllowedOnRDN',
    68: 'entryAlreadyExists',
    69: 'objectClassModsProhibited',
    70: 'resultsTooLarge',
    71: 'affectsMultipleDSAs',
    76: 'virtualListViewError or controlError',
    80: 'other',
    81: 'serverDown',
    82: 'localError',
    83: 'encodingError',
    84: 'decodingError',
    85: 'timeout',
    86: 'authUnknown',
    87: 'filterError',
    88: 'userCanceled',
    89: 'paramError',
    90: 'noMemory',
    91: 'connectError',
    92: 'notSupported',
    93: 'controlNotFound',
    94: 'noResultsReturned',
    95: 'moreResultsToReturn',
    96: 'clientLoop',
    97: 'referralLimitExceeded',
    100: 'invalidResponse',
    101: 'ambiguousResponse',
    112: 'tlsNotSupported',
    113: 'intermediateResponse',
    114: 'unknownType',
    118: 'canceled',
    119: 'noSuchOperation',
    120: 'tooLate',
    121: 'cannotCancel',
    122: 'assertionFailed',
    123: 'authorizationDenied',
    4096: 'e-syncRefreshRequired',
    16654: 'noOperation',
}


class ActiveDirectory:
    ldap = None
    base_dn = None

    def __init__(self):
        self.page_control = SimplePagedResultsControl(True, size=1000, cookie='')

    def connect(self, dc, username, password, base_dn=None, ldaps=False, port=None):
        if ldaps:
            protocol = 'ldaps'
        else:
            protocol = 'ldap'

        if protocol == 'ldaps' and port is None:
            port = 636
        else:
            port = 389

        self.base_dn = base_dn

        ldap.set_option(ldap.OPT_REFERRALS, 0)
        ldap.set_option(ldap.OPT_NETWORK_TIMEOUT, 2)
        ldap.set_option(ldap.OPT_SIZELIMIT, 2500)

        self.ldap = ldap.initialize('%s://%s:%d' % (protocol, dc, port), bytes_mode=False)
        self.ldap.simple_bind_s(username, password)

    def ldap_query(self, query: str, base_dn: str, single_result=True, subtree=True,
                   attributes: list = None, pagination=False,
                   result_class: Type[ADObject] = None) -> Union[List[Dict], List[ADObject]]:

        if not attributes:
            attributes = ['dn']
        if subtree:
            scope = ldap.SCOPE_SUBTREE
        else:
            scope = ldap.SCOPE_ONELEVEL

        if not pagination:
            entries = self.ldap.search_s(base_dn, scope, query, attributes)
        else:
            return paginated_query(self.ldap, attributes, base_dn, scope, query, result_class)

        """if len(attributes) == 1:
            return entries[1]"""

        return entries

    def group_members(self, group_dn):
        members = self.ldap_query('(distinguishedName=%s)' % group_dn, attributes=['member'],
                                  single_result=False, base_dn=self.base_dn)
        return members[0][1]['member']

    def organizational_units(self, base_dn=None, subtree=True):
        if not base_dn:
            base_dn = self.base_dn
        return self.ldap_query('(objectClass=organizationalUnit)', base_dn, single_result=False, subtree=subtree)



# Generated by Django 3.1 on 2020-09-18 11:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ad_import', '0002_auto_20200917_2001'),
    ]

    operations = [
        migrations.CreateModel(
            name='Workstation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cn', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='beskrivelse')),
                ('distinguishedName', models.CharField(blank=True, max_length=300, null=True, verbose_name='DN')),
                ('lastLogon', models.DateTimeField(blank=True, null=True, verbose_name='siste pålogging')),
                ('logonCount', models.IntegerField(null=True, verbose_name='Antall pålogginger')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='navn')),
                ('objectGUID', models.CharField(blank=True, max_length=255, null=True)),
                ('objectSid', models.CharField(blank=True, max_length=255, null=True)),
                ('operatingSystem', models.CharField(blank=True, max_length=255, null=True, verbose_name='Operativsystem')),
                ('operatingSystemHotfix', models.CharField(blank=True, max_length=255, null=True)),
                ('operatingSystemServicePack', models.CharField(blank=True, max_length=255, null=True)),
                ('operatingSystemVersion', models.CharField(blank=True, max_length=255, null=True)),
                ('pwdLastSet', models.DateTimeField(null=True, verbose_name='passord sist endret')),
                ('userAccountControl', models.IntegerField(null=True)),
                ('whenChanged', models.DateTimeField(null=True, verbose_name='sist endret')),
                ('whenCreated', models.DateTimeField(null=True, verbose_name='opprettet')),
                ('last_update', models.DateTimeField(auto_now=True, verbose_name='sist oppdatert')),
                ('directory', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='ad_import.directory')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Server',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cn', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='beskrivelse')),
                ('distinguishedName', models.CharField(blank=True, max_length=300, null=True, verbose_name='DN')),
                ('lastLogon', models.DateTimeField(blank=True, null=True, verbose_name='siste pålogging')),
                ('logonCount', models.IntegerField(null=True, verbose_name='Antall pålogginger')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='navn')),
                ('objectGUID', models.CharField(blank=True, max_length=255, null=True)),
                ('objectSid', models.CharField(blank=True, max_length=255, null=True)),
                ('operatingSystem', models.CharField(blank=True, max_length=255, null=True, verbose_name='Operativsystem')),
                ('operatingSystemHotfix', models.CharField(blank=True, max_length=255, null=True)),
                ('operatingSystemServicePack', models.CharField(blank=True, max_length=255, null=True)),
                ('operatingSystemVersion', models.CharField(blank=True, max_length=255, null=True)),
                ('pwdLastSet', models.DateTimeField(null=True, verbose_name='passord sist endret')),
                ('userAccountControl', models.IntegerField(null=True)),
                ('whenChanged', models.DateTimeField(null=True, verbose_name='sist endret')),
                ('whenCreated', models.DateTimeField(null=True, verbose_name='opprettet')),
                ('last_update', models.DateTimeField(auto_now=True, verbose_name='sist oppdatert')),
                ('directory', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='ad_import.directory')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from abc import ABC

from ad_import.load_data import LoadAd
from ad_import.models import Computer


class LoadComputers(LoadAd, ABC):
    fields = ['cn',
              'description',
              'distinguishedName',
              'lastLogon',
              'logonCount',
              'name',
              'objectGUID',
              'objectSid',
              'operatingSystem',
              'operatingSystemHotfix',
              'operatingSystemServicePack',
              'operatingSystemVersion',
              'pwdLastSet',
              'userAccountControl',
              'whenChanged',
              'whenCreated',
              ]

    model = Computer

    def load(self, query):
        entries = self.run_query(query)
        for user_data in entries:
            computer: Computer
            computer, exist = self.get_object(user_data)

            computer.cn = user_data.string('cn')
            computer.description = user_data.string('description')
            computer.distinguishedName = user_data.string('distinguishedName')
            computer.lastLogon = user_data.date('lastLogon')
            computer.logonCount = user_data.numeric('logonCount')
            computer.name = user_data.string('name')
            computer.objectGUID = user_data.bytes('objectGUID')
            computer.objectSid = user_data.bytes('objectSid')
            computer.operatingSystem = user_data.string('operatingSystem')
            computer.operatingSystemHotfix = user_data.string('operatingSystemHotfix')
            computer.operatingSystemServicePack = user_data.string('operatingSystemServicePack')
            computer.operatingSystemVersion = user_data.string('operatingSystemVersion')
            computer.pwdLastSet = user_data.date('pwdLastSet')
            computer.userAccountControl = user_data.numeric('userAccountControl')
            computer.whenChanged = user_data.date_string('whenChanged')
            computer.whenCreated = user_data.date_string('whenCreated')

            if computer.disabled() and not exist:
                print('%s is disabled' % computer)
            else:
                computer.save()



from abc import ABC, abstractmethod
from typing import List

from django.db.models import Model, QuerySet

from ad_import.ad import ADObject, ActiveDirectory
from ad_import.models import Directory, Query


class LoadAd(ABC):
    directory: Directory
    model: Model
    fields = []

    def __init__(self):
        self.ad = ActiveDirectory()

    def connect(self, directory_name):
        directory = Directory.objects.get(name=directory_name)
        self.directory = directory
        self.ad.connect(dc=directory.dc,
                        username=directory.username,
                        password=directory.password,
                        base_dn=directory.dn,
                        ldaps=directory.ldaps)

    @staticmethod
    def base_dn(query: Query) -> str:
        if query.base_dn:
            return query.base_dn
        else:
            return query.directory.dn

    def get_object(self, entry: ADObject):
        """
        Get django object for a LDAP entry
        Use SID to check for existing objects in database
        :param entry:
        :return:
        """
        try:
            user = self.model.objects.get(directory=self.directory, objectSid=entry.bytes('objectSid'))
            exist = True
        except self.model.DoesNotExist:
            user = self.model(directory=self.directory)
            exist = False

        return user, exist

    @abstractmethod
    def load(self, query: Query):
        pass

    @property
    def queries(self) -> QuerySet[Query]:
        """
        Get queries from the current directory
        :return: Queries
        """
        return self.directory.queries.filter(target=self.model.__module__)

    def run_query(self, query: Query) -> List[ADObject]:
        base_dn = self.base_dn(query)
        return self.ad.ldap_query(query.query,
                                  base_dn,
                                  single_result=False,
                                  subtree=True,
                                  pagination=True,
                                  attributes=self.fields,
                                  result_class=ADObject)

    @staticmethod
    def is_disabled(entry: ADObject):
        if entry.field('userAccountControl') is None:
            return None

        return entry.numeric('userAccountControl') & 2 == 2  # Check if account is disabled



from django.db import models


class Directory(models.Model):
    name = models.CharField(max_length=100, null=True, blank=True)
    dn = models.CharField(max_length=100)
    dc = models.GenericIPAddressField()
    username = models.CharField(max_length=100)
    password = models.CharField(max_length=100)
    ldaps = models.BooleanField(default=False)

    def __str__(self):
        return self.dn

    class Meta:
        # verbose_name = 'Directory'
        verbose_name_plural = 'directories'


from django.db import models

from .directory import Directory


class Computer(models.Model):
    cn = models.CharField(max_length=255, blank=True, null=True)
    description = models.CharField('beskrivelse', max_length=255, blank=True, null=True)
    distinguishedName = models.CharField('DN', max_length=300, blank=True, null=True)
    lastLogon = models.DateTimeField('siste pålogging', null=True, blank=True)
    logonCount = models.IntegerField('Antall pålogginger', null=True)
    name = models.CharField('navn', max_length=255, blank=True, null=True)
    objectGUID = models.CharField(max_length=255, blank=True, null=True)
    objectSid = models.CharField(max_length=255, blank=True, null=True)
    operatingSystem = models.CharField('Operativsystem', max_length=255, blank=True, null=True)
    operatingSystemHotfix = models.CharField(max_length=255, blank=True, null=True)
    operatingSystemServicePack = models.CharField(max_length=255, blank=True, null=True)
    operatingSystemVersion = models.CharField(max_length=255, blank=True, null=True)
    pwdLastSet = models.DateTimeField('passord sist endret', null=True)
    userAccountControl = models.IntegerField(null=True)
    whenChanged = models.DateTimeField('sist endret', null=True)
    whenCreated = models.DateTimeField('opprettet', null=True)

    directory = models.ForeignKey(Directory, on_delete=models.PROTECT)
    last_update = models.DateTimeField('sist oppdatert', auto_now=True)

    class Meta:
        abstract = True

    def __str__(self):
        return '%s' % self.name

    def disabled(self):
        return self.userAccountControl & 2 == 2



from django.contrib import admin

# Register your models here.
from ad_import.models import User, Directory, Query, Server, Workstation


@admin.register(Directory)
class DirectoryAdmin(admin.ModelAdmin):
    list_display = ['dc', 'dn']


@admin.register(Query)
class QueryAdmin(admin.ModelAdmin):
    list_display = ['name', 'directory', 'query', 'base_dn', 'type', 'target']
    list_filter = ['directory', 'type', 'target']
    save_as = True


@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    list_display = ['directory', 'displayName', 'givenName', 'sn', 'sAMAccountName']
    list_filter = ['directory']


class ComputerAdmin(admin.ModelAdmin):
    list_display = ['directory', 'cn', 'operatingSystem']
    list_filter = ['directory', 'operatingSystem', 'operatingSystemVersion']


@admin.register(Server)
class ServerAdmin(ComputerAdmin):
    pass


@admin.register(Workstation)
class WorkstationAdmin(ComputerAdmin):
    pass



from datetime import datetime

from ad_import.utils.ad_utils import parse_date


class ADObject:
    def __init__(self, user):
        self.object = user[1]
        self.dn = user[0]

    def field(self, field: str):
        if field in self.object:
            return self.object[field][0]

    def string(self, field) -> str:
        try:
            data = self.field(field)
            if data:
                return data.decode('utf-8')
        except UnicodeDecodeError as e:
            print('Error decoding field %s' % field)
            raise e

    def numeric(self, field) -> int:
        try:
            data = self.field(field)
            if data:
                return int(data)
        except ValueError as e:
            print('Error decoding field %s' % field)
            raise e

    def bytes(self, field) -> int:
        if field in self.object:
            return int.from_bytes(self.field(field), 'big')

    def hex(self, field) -> int:
        if field in self.object:
            return int(self.field(field), 16)

    def date(self, field) -> datetime:
        data = self.numeric(field)
        if data:
            return parse_date(data)

    def date_string(self, field) -> datetime:
        data = self.string(field)
        # 2020 09 09 08:56:44.0Z
        return datetime.strptime(data, '%Y%m%d%H%M%S.0Z')
